#!/usr/bin/perl

=pod 

=head1 NAME

convert_rnammer_gff3.pl

=head1 SYNOPSIS

convert_rnammer_gff3.pl --infile rnammer.gff --outfile rnammer.rnammer.gff3 [--add_parent type]

=head1 DESCRIPTION

Convert rnammer in GFF3 format (http://www.sequenceontology.org/gff3.shtml)

=cut

BEGIN
{
	my ($dir,$file) = $0 =~ /(.+)\/(.+)/;
	unshift (@INC,"$dir/../../corelib");
	unshift (@INC,"$dir/../../lib");
}

use strict; 
use warnings;
use Carp;

use ParamParser;
use Logger;
use General;

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG', "infile=s", "outfile=s", "add_parent=s");

	$o_param->SetUsage(my $usage=sub { &Usage(); } );
	$o_param->AssertFileExists('infile');
	$o_param->AssertDefined('outfile');

	my $infile  = $o_param->Get('infile');
	my $outfile = $o_param->Get('outfile');
	my $f_in    = &General::GetStreamIn($infile);
	my $f_out   = &General::GetStreamOut($outfile);

	# Skip the first three lines
	my $line = <$f_in>;
	$line    = <$f_in>;
	$line    = <$f_in>;
	# Print gff3 header
	print $f_out "##gff-version 3\n";

	while ( my $line = <$f_in> )
	{
		chomp($line);
        next if ( $line =~ /^#/ || $line eq '' );
		#Mt35Chr5        RNAmmer-1.2     rRNA    25254354        25254468        50.3    +       .       8s_rRNA
        my @a_ = split(/\t/, $line);
		$a_[0] =~ s/ /_/;
		my ($beg, $end, $strand) = ($a_[3],$a_[4],$a_[6]);
		my $type = $a_[8];
		my $id     = "$a_[0]-rRNA-$type-$beg-$end";
		if ( $o_param->IsDefined('add_parent') )
		{
			my $parent_type = $o_param->Get('add_parent');
			print $f_out "$a_[0]\t$a_[1]\t$parent_type\t$beg\t$end\t$a_[5]\t$strand\t.\tID=P_$id;Name=$id;Ontology_term=SO:0000252;subunit=$type\n";
			print $f_out "$a_[0]\t$a_[1]\t$a_[2]\t$beg\t$end\t$a_[5]\t$strand\t.\tID=$id;Name=$id;Parent=P_$id;Ontology_term=SO:0000252;subunit=$type\n";
		}
		else
		{
			print $f_out "$a_[0]\t$a_[1]\t$a_[2]\t$beg\t$end\t$a_[5]\t$strand\t.\tID=$id;Name=$id;Ontology_term=SO:0000252;subunit=$type\n";
		}
	}
	$f_in->close;
	$f_out->close;
}

sub Usage
{
print STDERR<<END
$0 
	--help
	--infile     filename   gff rnammer output format (gzipped allowed)
	--outfile    filename   gff3 format 
[Optional]
	--add_parent string     If defined, add systematically a parent element from this type. (column 3 in GFF3) Examples: 'gene', 'ncRNA'
END
;
}
