#!/usr/bin/perl

BEGIN
{
    $ENV{LANG} = "C";
}

use FindBin;
use lib "$FindBin::Bin/../../lib/";
use lib "$FindBin::Bin/../../corelib/"; 


use strict;
use warnings;
use General;
use GeneralBioinfo;
use ParamParser;

my $DEBUG;

MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',\&Usage,'help','pcs=i','pci=i','in=s','out=s',
								  'format=s','span=i', 'max_intron_length=i', 'best','min_length_short_unspliced=i', 'seqin=s', 'debug');

	$o_param->SetUnlessDefined('in','stdin');
	$o_param->SetUnlessDefined('out','stdout');
	$o_param->SetUnlessDefined('pci',0);
	$o_param->SetUnlessDefined('pcs',0);
	$o_param->SetUnlessDefined('span',100);
	$o_param->SetUnlessDefined('min_length_short_unspliced',0);
	$o_param->SetUnlessDefined('max_intron_length',100000);
	$o_param->SetUnlessDefined('format','gbrowse');
	$o_param->AssertAllowedValue('format','gbrowse','annotastruct','est');

	$DEBUG = 0;
	$DEBUG = 1 if($o_param->IsDefined('debug'));

	my $pci = $o_param->Get('pci');
	my $pcs = $o_param->Get('pcs');
	my $span = $o_param->Get('span');
	my $minlen_su = $o_param->Get('min_length_short_unspliced');

	my $format = $o_param->Get('format');
	my $max_intron_length = $o_param->Get('max_intron_length');

	my $best = 0;
	$best    = 1 if ($o_param->IsDefined('best'));

	my %h_seq;
	if ($o_param->IsDefined('seqin'))
	{
		$o_param->AssertFileExists('seqin');
		&FastaToHash($o_param->Get('seqin'), \%h_seq, 1);
	}
	
	my $f_in  = $o_param->GetStreamIn('in');
	my $f_out = $o_param->GetStreamOut('out');
	my $f_out_rm = undef;
	if ($minlen_su > 0)
	{
		my $unspliced = $o_param->Get('out');
		$unspliced =~ s/.gff3$//;
		$unspliced .= ".rm_unspliced_${minlen_su}.gff3";
		$f_out_rm = &General::GetStreamOut($unspliced);
	}
	
	my %h_val=();
	my %h_prevval=();
	my @a_hit = ();
	my $ok=&FALSE;
	my $remove_previous = 0; # boolean used to keep only best hit (if --best activated)
	
	# au format -f gff3_gene tous les block gene sont separes par ### 
    # et les exons sont ordonnes avec 1er exon au debut et dernier a la fin
	# il y a des CDS de 1 nt, faut'il les virer ?
	# je vais creer les introns et rajouter le type SO
	my $cpt_intron = 0;
	
	# Key:mrna, value=Identity percentage
	my %h_hit_id;
	# Key:mrna, value=Coverage percentage
	my %h_hit_cov;
	# Key:mrna_start_end_strand
	my %h_hit_pos;

	# Key:mrna, value=lines 
	my %h_hit;
	my %h_hit_rm;

	my @a_mrna_id;
	my @a_mrna_id_rm;

	my $mrna_name;

	while(my $l=<$f_in>)
	{
		%h_val = ();
		chomp($l);
		next if ($l =~ /^# / || $l =~ /^##gff-version/);

		if ( $l =~ /^###/ && scalar(@a_hit) )
		{
			print "DUMP $mrna_name\n" if ($ok && $DEBUG);
			&Dump(\@a_hit, $mrna_name, $format, \%h_hit, \@a_mrna_id, $f_out, $best, $minlen_su, $remove_previous, \%h_hit_rm, \@a_mrna_id_rm, $f_out_rm) if ($ok);
			
			# Reinit
			@a_hit         = ();
			$ok            = &FALSE;
			%h_prevval     = ();
			$mrna_name     = "";
			$remove_previous = 0;
			next;
		}
		
		&ParseGFF3Line(\$l,2,\%h_val);
		
		# Filtering PCS, PCI and best 
		if ($h_val{type} eq 'mRNA')
		{
			$mrna_name = $h_val{Name};
			my $mrna_start  =  $h_val{start};
			my $mrna_end    =  $h_val{end};
			my $mrna_strand =  $h_val{strand};
			my $coverage = 0;
			my $identity = 0;
			$coverage = $h_val{Coverage} if exists($h_val{Coverage});
			$coverage = $h_val{coverage} if exists($h_val{coverage});
			$identity = $h_val{Identity} if exists($h_val{Identity});
			$identity = $h_val{identity} if exists($h_val{identity});
			
			# the gene is kept only if its mrna minlen, minpci and minpcs are ok
			$ok = &TRUE if ($coverage >= $pcs && $identity >= $pci && $h_val{len} >= $span && 
							$mrna_start > 0 );

			# If the seq len is known, check the mrna end is lower to the the seq len
			if ( $ok && defined($h_seq{$h_val{seqid}}) )
			{
				$ok = ($mrna_end <= $h_seq{$h_val{seqid}}{len}) ? &TRUE : &FALSE;
			}
			
			# this mRNA was already view: keep only the best
			if ($best && $ok && defined ($h_hit{$mrna_name}))
			{
				$ok = &FALSE;
				
                # We found a best hit. Remove previous one
				if ( ($coverage > $h_hit_cov{$mrna_name}) || 
					 ($coverage == $h_hit_cov{$mrna_name} && $identity > $h_hit_id{$mrna_name}))
				{
					print "BEST HIT FOUND for $mrna_name (cov $coverage, id $identity): remove previous one\n" if ($DEBUG);
					$remove_previous = 1;
					$ok = &TRUE;
				}
				# We found an other best hit: dont remove previous one
				elsif ( ($coverage == $h_hit_cov{$mrna_name}) && 
						($identity == $h_hit_id{$mrna_name}))
				{
					# Keep if this mrna was not previously mapped at this position 
					$ok = &TRUE if (!defined($h_hit_pos{"$mrna_name.$mrna_start.$mrna_end.$mrna_strand"}));			
				}					
			}
			# Update variables
			if ($ok)
			{
				$h_hit_id{$mrna_name}  = $identity;
				$h_hit_cov{$mrna_name} = $coverage;
				$h_hit_pos{"$mrna_name.$mrna_start.$mrna_end.$mrna_strand"} = 1;
			}
		}
			
		if ( $format eq 'annotastruct')
		{
			next if ( ( $h_val{type} eq 'gene') ||  ($h_val{type} eq 'mRNA'));

			$l =~ s/Parent=[^\;]+// if (  $format eq 'annotastruct' );

			# Extract the intron
			if ( $h_val{type} eq 'exon' && defined($h_prevval{type}) && $h_prevval{type} eq 'exon' )
			{
				my $start  = ($h_val{strand} eq '+')  ? $h_prevval{end}+1 : $h_val{end}+1;
				my $end    = ($h_val{strand} eq '+')  ? $h_val{start}-1   : $h_prevval{start}-1;
				my $len    = $end - $start + 1;
				my $intron = "$h_val{seqid}\t$h_val{source}\tintron\t$start\t$end\t$len\t$h_val{strand}\t.\tID=$h_val{seqid}.$cpt_intron;Ontology_term=SO:0000188";
				$cpt_intron++;

				$ok = &FALSE if ($len > $max_intron_length);

				if ( $len > 0 ) # il faudrait mettre > 40 ?
				{
					push(@a_hit, $intron);
				}
			}
		}
		elsif ( $format eq 'est')
		{
			next if ( ( $h_val{type} eq 'gene') ||  ($h_val{type} eq 'CDS'));
			$l =~ s/\tmRNA\t/\tEST_match\t/;
			$l =~ s/\texon\t/\tmatch_part\t/;
			$l =~ s/Parent=[^\;]+// if ( $h_val{type} eq 'mRNA' );

			# Check the maximum intron length
			if ( $h_val{type} eq 'exon' && defined($h_prevval{type}) && $h_prevval{type} eq 'exon' )
			{
				my $start  = ($h_val{strand} eq '+')  ? $h_prevval{end}+1 : $h_val{end}+1;
				my $end    = ($h_val{strand} eq '+')  ? $h_val{start}-1   : $h_prevval{start}-1;
				my $len    = $end - $start + 1;
				$ok = &FALSE if ($len > $max_intron_length);
			}
		}

		# Save the line
		push(@a_hit, $l);
			
		# current --> prev
		%h_prevval = ();
		%h_prevval = %h_val;
	}

	&Dump(\@a_hit, $mrna_name, $format, \%h_hit, \@a_mrna_id, $f_out, $best,$minlen_su, $remove_previous, \%h_hit_rm, \@a_mrna_id_rm, $f_out_rm) if ($ok);
	$f_in->close;
	
	# write the final result in the file
	if ($best)
	{
		&SaveInFile($f_out,    \%h_hit,    \@a_mrna_id);
		&SaveInFile($f_out_rm, \%h_hit_rm, \@a_mrna_id_rm) if ($minlen_su > 0);
	}

	$f_out->close;
	$f_out_rm->close if (defined $f_out_rm);
	
}


# Format the hit for the output
# If --best, save the results in  $rh_hits, else
# Write directly the result in the file $f_out 
#
# $ra_hit    Ref to an array. GFF3 lines describing the mrna
# mrna_name  Name of the gene
# $format    est or annotastruct
# $rh_hits   Ref to a hash: key: mrna_name, value : line describing the mrna
# $ra_mrna   Array used to keep the same order
# $f_out     File descriptor
# $best      If true fill $rh_hits and $ra_mrna. Else print directly in the file
# 
sub Dump
{
	my ($ra_hit, $mrna_name, $format, $rh_hits, $ra_mrna, $f_out, $best, $minlen_su, $remove_previous, $rh_hits_rm, $ra_mrna_rm, $f_out_rm) = @_;

	my %h_val=();
	
	my $too_short = 0;
	# count the exon and CDS number
	my @a_exons   = grep(/\t(exon|match_part)\t/,@$ra_hit);
	my $nb_exons  = scalar(@a_exons);
	my $nb_cds    = grep(/\tCDS\t/,  @$ra_hit);

	if ( $minlen_su > 0  && $nb_exons < 2 ) 
	{
		&ParseGFF3Line(\$a_exons[0], 2, \%h_val);
		if ( ($h_val{end} - $h_val{start} + 1) < $minlen_su )
		{
			print STDERR "Unspliced too short: ".join("\n",@$ra_hit)."\n" if ($DEBUG);
			$too_short = 1;
		}
	}

	# Add to this block the appropriate ontology terms
	my $cpt_exon = 0;
	my $cpt_cds  = 0;
	for(my $i=0; $i<scalar(@$ra_hit); $i++)
	{
		next if ( $$ra_hit[$i] =~ /^#/ );
		&ParseGFF3Line(\$$ra_hit[$i], 2, \%h_val);

		if ($h_val{type} eq 'exon' )
		{
			$cpt_exon++;
			if ( $cpt_exon == 1 ) # First exon
			{
				if ( $nb_exons == 1 )
				{
				
					$$ra_hit[$i] =~ s/\texon\t/\ttranscript_region\t/ if ( $format ne 'gbrowse' );
					$$ra_hit[$i] .= ";Ontology_term=SO:0000833";
				}
				else
				{
					$$ra_hit[$i] .= ";Ontology_term=SO:0000200";
				}
			}
			elsif ( $cpt_exon == $nb_exons )  #Last  exon
			{
				$$ra_hit[$i] .= ";Ontology_term=SO:0000202";
			}
			else
			{
				$$ra_hit[$i] .= ";Ontology_term=SO:0000147";
			}
		}
		elsif ($h_val{type} eq 'CDS' )
		{
			$cpt_cds++;
			if ($cpt_cds == 1 )
			{
				if ($nb_cds == 1 )
				{
					# sngle  CDS 
					$$ra_hit[$i] .= ";Ontology_term=SO:0005845";
				}
				else
				{
					# first CDS SO:0000196
					$$ra_hit[$i] .= ";Ontology_term=SO:0000196";
				}
			}
			elsif ($cpt_cds == $nb_cds) # last cds 
			{
				$$ra_hit[$i] .= ";Ontology_term=SO:0000197";
			}
			else
			{
				# interior cds 
				$$ra_hit[$i] .= ";Ontology_term=SO:0000004";
			}
		}
		$$ra_hit[$i] .= "\n";
	}
	
	# Case unspliced transcript and a short length
	if ($too_short)
	{
		print STDERR "Skipped: ".join("\n", @$ra_hit) . "\n" if ($DEBUG);
		if ($best)
		{
			push(@$ra_mrna_rm, $mrna_name) if (!defined $$rh_hits_rm{$mrna_name});
			$$rh_hits_rm{$mrna_name} .= join("", @$ra_hit);
		}
		else
		{
			print $f_out_rm @$ra_hit;
		}
	}
	else
	{
		if ($best)
		{
			# This hit is better than the previous ones: so remove them
			$$rh_hits{$mrna_name} = "" if ($remove_previous == 1);
			# Fill the hash and the array (to keep sorting)
			push(@$ra_mrna, $mrna_name) if (!defined $$rh_hits{$mrna_name});
			$$rh_hits{$mrna_name} .= join("", @$ra_hit);
		}
		else
		{
			print $f_out @$ra_hit;
		}
	}
	
	return;
}


sub SaveInFile
{
	my ($f_out, $rh_hit, $ra_mrna_id) = @_;
	
	foreach my $elt (@$ra_mrna_id)
	{
		print $f_out $$rh_hit{$elt};
	}

	return;
}


sub Usage
{
print STDERR<<END
$0
	--help
	--in     gmapfile.gff3   Important: a gene is defined in a block separated by two lines '###'
	--out    filtered.gff3
[Optional]
	--pci    integer             [0] minimum percentage identity
	--pcs    integer             [0] minimum percentage coverage
	--span    integer            [100] minimum genome span
	--max_intron_length integer  [100000] maximum length of an intron
	--format string              [gbrowse]   gbrowse|annotastruct|est
	--best                       Keep all best hits only (maximize the coverage and the identity)
	--min_length_short_unspliced integer [0] remove short unspliced matches. Save them in a separate file
    --seqin  fasta/multifasta    If defined, Check coordinates of gmap results are not upper to the seqlen
    --debug
END
}
