#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=head1 NAME


=head1 SYNOPSIS



=head1 DESCRIPTION


=cut

BEGIN
{
    $ENV{LANG} = "C";

    use Cwd 'abs_path';
    use File::Basename;
    my ($dirprg) = dirname(abs_path($0));
    unshift(@INC, "$dirprg/../../../lib");
    unshift(@INC, "$dirprg/../../../corelib");

}

use strict;
use warnings;
use Carp;
use ParamParser;
use General;
use Cwd;

my $DIRNAME = "MyGenomeBrowser";

MAIN:
{
    my $o_param = New ParamParser(
                                  'GETOPTLONG', \&Usage, 'egnep_outdir=s', 'cfg=s', 'egnep_workdir=s', 'output_suffix=s'
                                  );

    $o_param->SetBehaviour('ignore_space');
    $o_param->Update('ENV', "A");
    $o_param->AssertFileExists('cfg');
    $o_param->AssertFullPath('cfg');
    $o_param->Update($o_param->Get('cfg'), 'A');
    $o_param->SetBehaviour('use_substitution_table');
    $o_param->SetSubstitution('%i', $o_param->Get('EGNEP'));
    $o_param->SetSubstitution('%e', $o_param->Get('EUGENEDIR'));
    $o_param->Update($o_param->Get('cfg'), 'U');
    $o_param->AssertDirExists('egnep_outdir');
    my $egnepdir = $o_param->Get('egnep_outdir');
    $o_param->SetUnlessDefined('egnep_workdir', $egnepdir);
    $o_param->AssertDirExists('egnep_workdir');
    my $egnepworkdir = $o_param->Get('egnep_workdir');
    my $fof      = "$egnepworkdir/entries.fof";
	my $cfg          = $o_param->Get('cfg');
    $o_param->Set('fof', $fof);
    $o_param->AssertFileExists('fof');	
	$o_param->AssertDefined('output_prefix');
	
	my @a_localtime  = localtime;
	my ($year, $month, $day) = ($a_localtime[5] + 1900, $a_localtime[4] + 1, $a_localtime[3]);
	$month = ( $month < 10 ) ? "0$month": $month;
	$day = ( $day < 10 ) ? "0$day": $day;
	my $date = "$year$month$day";	
	
	$o_param->SetUnlessDefined('output_suffix', $date);
	my $output_suffix = "";
	$output_suffix    = ".".$o_param->Get('output_suffix') if ($o_param->Get('output_suffix') ne "");
	
	my $prefix    = $o_param->Get('output_prefix');
	my $outprefix = $prefix;
	$outprefix   .= $output_suffix;
	
    # Load sequences	
    my @a_seq;
    my $f_fof = $o_param->GetStreamIn('fof');
    while (my $seq = <$f_fof>)
    {
		chomp($seq);
		
		if (!-f $seq)
		{
			print STDERR "ERROR: sequence $seq read in $fof does not exist.\n";
			exit;
		}
		push(@a_seq, $seq);
    }
    $f_fof->close;


	if (!-e "$egnepdir/$prefix" || !-e "$egnepdir/$prefix.gff3")
	{
		print STDERR "ERROR: the sequence or the annotation file is missing in $egnepdir\n";
		exit;
	}

	$DIRNAME .= $output_suffix;
	
	# create a directory for mygenomebrowser
	my $outdir = cwd()."/$DIRNAME";
    system ("rm $DIRNAME.tar.gz") if (-e "$DIRNAME.tar.gz");
    system ("rm -rf $outdir")     if (-d $outdir);
    mkdir $outdir if ( ! -d $outdir);
    
    my $summary = "$outdir/README";
    my $f_readme = &General::GetStreamOut($summary);
	print $f_readme "File\tDescription\n";
	
    # copy the sequence and the annotation
	system("cp $egnepdir/$prefix $outdir/$outprefix");
	system("cp $egnepdir/$prefix.gff3 $outdir/$outprefix.gff3");
	# copy the egnep configuration file
   	system("cp $cfg $outdir/$outprefix.cfg");
	# copy the eugene par file
	system("cp $egnepworkdir/egn_param/eugenev1.par $outdir/$outprefix.par");
	# copy the IMM matrices
	system("cp $egnepworkdir/egn_param/eugene.mat $outdir/$outprefix.mat");
	print $f_readme "$outprefix\tGenome\n";	
	print $f_readme "$outprefix.gff3\tStructural annotation\n";
	print $f_readme "$outprefix.cfg\tConfiguration file\n";
	print $f_readme "$outprefix.par\tEuGene parameter file\n";
	print $f_readme "$outprefix.mat\tEuGene IMM matrix\n";
	
    # Get the blastX files
    my $list = $o_param->Get('blastx_db_list');    # "2 5 6"
    my @a_db_nb = split(/\s+/, $list);
    foreach my $nb (@a_db_nb)
    {
        my $db = $o_param->Get("blastx_db_$nb\_file");
        $db =~ s/\s+$//;
	my @a_db_files;
	foreach my $seq (@a_seq)
	{
	    push(@a_db_files, $seq.".blast$nb.gff3");	
	}
	&CatFiles(\@a_db_files, $outdir."/$outprefix.blast$nb.gff3");
	print $f_readme "$outprefix.blast$nb.gff3\t$db\n";
    }
    
    # Get the est files
    $list = $o_param->Get('est_list');    # "2 5 6"
    @a_db_nb = split(/\s+/, $list);
	
	push (@a_db_nb, 1000) if ( $o_param->IsDefined('gmap_intron_filter') && $o_param->Get('gmap_intron_filter') == 1);
	
    foreach my $nb (@a_db_nb)
    {
        my $db = $o_param->Get("est_$nb\_file");
        $db =~ s/\s+$//;
	my @a_db_files;
	my @a_db_unspliced_files;
	foreach my $seq (@a_seq)
	{
	    push(@a_db_files,            $seq.".est$nb.gff3");	
	    push (@a_db_unspliced_files, $seq.".est$nb.rm_unspliced_10000000.gff3") if (-e  $seq.".est$nb.rm_unspliced_10000000.gff3");
	}
		&CatFiles(\@a_db_files, $outdir."/$outprefix.est$nb.gff3");
		my $note = "";
		$note = "\tAfter GMap intron filtering" if ($nb == 1000);
	print $f_readme "$outprefix.est$nb.gff3\t$db$note\n";
	
	if (scalar(@a_db_unspliced_files) > 0)
	{
	    &CatFiles(\@a_db_unspliced_files, $outdir."/$outprefix.est$nb.rm_unspliced_10000000.gff3");
	    print $f_readme "$outprefix.est$nb.rm_unspliced_10000000.gff3\t$db\tUnspliced alignments$note\n";
	}
    }
    
    # Get the ncRNA files
    my @a_ncrna;
    foreach my $seq (@a_seq)
    {
	push(@a_ncrna, $seq.".rfamscan.gff3");
	push(@a_ncrna, $seq.".rnammer.gff3");
	push(@a_ncrna, $seq.".trnascan.gff3");
    }
    &CatFiles(\@a_ncrna, $outdir."/$outprefix.ncRNA.gff3");
    print $f_readme "$outprefix.ncRNA.gff3\tKnown ncRNAs\n";	
    
	# Get the repet file
	my @a_rep;
	foreach my $seq (@a_seq)
	{
		push(@a_rep, $seq.".repet_noexpressed_nosimprot.gff3");
	}
	&CatFiles(\@a_rep, $outdir."/$outprefix.repet_noexpressed_nosimprot.gff3");
	print $f_readme "$outprefix.repet_noexpressed_nosimprot.gff3\tMasked repeat regions\n";
	$f_readme->close;


	# Add the egnep configuration file
	
	
	my $c = "tar -cvzf $DIRNAME.tar.gz $DIRNAME/*";
	system($c);
	$c = "rm -rf $DIRNAME";
	system($c);
}



sub CatFiles
{
    my ($ra_file, $new) = @_;
	
    my $f_full = &General::GetStreamOut($new);
    foreach my $file (@$ra_file)
    {
		next if (!-e $file);
        my $f_ = &General::GetStreamIn($file);
        while (my $line = <$f_>)
        {
            print $f_full $line;
        }
        $f_->close;
    }
    $f_full->close;

    return;
}

sub Usage
{
    print STDERR<<END
$0
[Mandatory]

  --egnep_outdir     dir         EGNEP out directory
  --cfg              filename    Configuration file

[Optional]
  --egnep_workdir    dir         EGNEP work directory
  --output_suffix    string      [.YEARMONTHDAY] Suffix to add at the end of the output files. 
END
}
