#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=cut


BEGIN
{
	$ENV{LANG} = "C";

	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../../../corelib");
}

use strict;
use Carp;
use ParamParser;
use General;
use GeneralBioinfo;

my $DEF_PARENT = "EST_match";
my $DEF_CHILD  = "match_part";
MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG',\&Usage,'in=s', 'out=s', 'parent=s', 'child=s', 'mindiff=i', 'histo', 'keep_unspliced', 'noclean', 'verbose');

    $o_param->SetBehaviour('assert_empty_file_allowed', 1);
    $o_param->AssertFileExists('in');
    $o_param->AssertDefined('out');
    $o_param->SetUnlessDefined('parent', $DEF_PARENT); 
    $o_param->SetUnlessDefined('child',  $DEF_CHILD);
    $o_param->SetUnlessDefined('mindiff', 1);
    my $parent = $o_param->Get('parent');
    my $child  = $o_param->Get('child');
    my $out    = $o_param->Get('out');
    my $in     = $o_param->Get('in');
    my %h_val     =();
    my %h_prevval =();
    my $global_intron_nb = 0;

    my %h_intron_nr;
    my %h_transcrit;
    my %h_transcrit_list;

    my $f_in  = $o_param->GetStreamIn('in');
    my $f_out = &GetStreamOut( $out.".intron.gff3");
    
    # 1) extraire les introns
    while(my $l=<$f_in>)
    {
	chomp($l);
	&ParseGFF3Line(\$l,&TRUE,\%h_val);
	if ( $h_val{type} eq $parent ) # EST_match
	{
	    %h_prevval     = ();
	}
	else # match_part
	{
	    if ( $h_val{type} eq $child && defined($h_prevval{type}) && $h_prevval{type} eq $child )
	    {
		# new intron
		$global_intron_nb++;

		my $start  = $h_prevval{end}+1;
		my $end    = $h_val{start}-1;
		my $len    = $end - $start + 1;
		my $strand = $h_val{strand};
		my $chr    = $h_val{seqid};
		my $transcrit =  $h_val{Name};
		my $key    = "$chr\_$start\_$end\_$strand";

		my $intronname   = "$chr\_$transcrit\_$start\_$end\_$strand";
		my $intron       = "$h_val{seqid}\t$h_val{source}\tintron\t$start\t$end\t.\t$strand\t.\tID=$intronname;Name=$intronname;Ontology_term=SO:0000188;\n";
		print $f_out $intron;

		# je sauvegarde le transcrit auquel appartient l'intron
		$h_transcrit{$key} .= "$transcrit,";
		$h_intron_nr{$key}++;
		$h_transcrit_list{$transcrit}++;
	    }
	}
	%h_prevval = ();
	%h_prevval = %h_val;
    }
    $f_in->close;
    $f_out->close;


    # je cree le fichier gff3 des introns sans redondance
    my $intronnrfile = $out.".intron.nr.gff3";
    my $f_out = &GetStreamOut($intronnrfile);
    foreach my $k (keys %h_intron_nr)
    {
	my $occ_nb = $h_intron_nr{$k};
	my ($c,$start, $end, $strand) =  $k =~ /^(.+)_(\d+)_(\d+)_(.+)$/;
	my $name = "$occ_nb\_$k";
	my $line = "$c\teugene\tintron\t$start\t$end\t.\t$strand\t.\tID=$name;Name=$name\n";
	print $f_out $line;
    }
    $f_out->close;


    # intersection avec lui meme pour rechercher les incoherences
    my $cmd = "intersectBed -a $intronnrfile -b $intronnrfile -s -wa -wb > $intronnrfile.intersection";
    system($cmd);

    my %h_incoherent_intron; # key : intron name, value: number of incoherence with others intron
    my %h_incoherent_couple; # key : intron name couple

    $f_in  = &GetStreamIn( "$intronnrfile.intersection");
    $f_out = &GetStreamOut("$intronnrfile.intersection.name");
    while (my $line = <$f_in>)
    {
	chomp($line);
	my @a_ = split(/\t/, $line);
	my %h_1;
	my %h_2;
	&GeneralBioinfo::ParseGFF3Line(\join("\t", @a_[0..8]), 1, \%h_1);
	&GeneralBioinfo::ParseGFF3Line(\join("\t", @a_[9..17]), 1, \%h_2);

	# intersection avec soi meme, je pass
	next if ($h_1{Name} eq $h_2{Name}) ;

	#  sort the two introns according to number of occurences (first most abundant)
	my @a_sort = sort MySort (($h_1{Name},$h_2{Name}));	
	my $couple = join ("\t", @a_sort); 

        # cette paire a deja ete vu, je pass!	
	next if (exists($h_incoherent_couple{$couple}));	

	print $f_out "$couple\n";

	$h_incoherent_couple{$couple} = 1;
	$h_incoherent_intron{$h_1{Name}}++;
	$h_incoherent_intron{$h_2{Name}}++;
	
    }
    $f_in ->close; $f_out->close;

    # je trie les lignes par nombre d'occurences des introns, colonne 1 puis colonne 2
    my $cmd = "sort -s -k2,2nr $intronnrfile.intersection.name | sort -s -k1,1nr  > $intronnrfile.intersection.name.sort";
    system($cmd);
    

    my $mindiff = $o_param->Get('mindiff');
    my %h_incoherent_couple_nbdiff;
    my %h_toremove ;
    my $f_o = &GetStreamOut("$out.ratio");
    # je parse les paires 

    $f_in = &GetStreamIn("$intronnrfile.intersection.name.sort");
    while (my $line = <$f_in>)
    {
	chomp($line);
	my @a_ = split(/\t/, $line);

	my $name1 = $a_[0];
	my $name2 = $a_[1];
	my ($nb1) = $name1 =~ /^(\d+)_/;
	my ($nb2) = $name2 =~ /^(\d+)_/;
	print $f_o ($nb1/$nb2)."\n";
	
	
	# je ne regarde que si l'abondance est differente
	# et si la différence est sup a 3
	next if ($nb1 eq $nb2 ) ; # || ( abs($nb1 - $nb2) <= $mindiff) ; #|| 1;

	
	# je verifie que l'intron majoritaire n'a pas ete precedemment supprime !!
	next if (exists($h_toremove{$name1}));

	$h_incoherent_couple_nbdiff{$line} = 1;
	# le deuxieme est moins present que le premier, je le supprime
	$h_toremove{$name2}++;
    }
    $f_in ->close;
    $f_o->close;

    if ($o_param->IsDefined('histo'))
    {
	my $cmd = "/home/esallet/work/Mt/MtrunA17r5.0-ANR/annotation/bin/egn-euk/bin/lipm_tab2histo.pl --in $out.ratio --out $out.ratio.histo --max 50 --step 1 --col 1";
	system($cmd);
	print "View histo of ratios :  $out.ratio.histo\n" if ($o_param->IsDefined('verbose'));
    }
 
    # look for mrna containing intron to remove
    my %h_transcrit_toremove;
    foreach my $intron (keys (%h_toremove))
    {
	my $key = $intron;
	$key =~ s/^\d+_//;
	my $list = $h_transcrit{$key};
	$list =~ s/,$//;
	my @a_list = split(",",$list);
	foreach my $t (@a_list)
	{
	    $h_transcrit_toremove{$t}++;
	}
    }
    
    $f_out = &General::GetStreamOut($out.".listtranscrittoremove");
    print $f_out join ("\n", keys (%h_transcrit_toremove))."\n";
    $f_out->close;

    # filtrage
    $f_out = $o_param->GetStreamOut('out');
    $f_in  = $o_param->GetStreamIn('in');
    
    # 1) Je supprime les alignements si le transcrit a un intron incoherent
    my $keep = 1;
    while(my $l=<$f_in>)
    {
	chomp($l);
	&ParseGFF3Line(\$l,&TRUE,\%h_val);
	if ( $h_val{type} eq $parent ) # EST_match
	{
	    if ( exists $h_transcrit_toremove{$h_val{Name}} )
	    {
		$keep = 0;
	    }
	    else
	    {
		$keep = 1;
	    }
	}
	print $f_out "$l\n" if ($keep);
    }
    $f_out->close;
    $f_in->close;

    # copy  the second file if exists
    if ($o_param->IsDefined('keep_unspliced'))
    {
	my $unspliced = $in;
	$unspliced =~ s/\.gff3$//;
	$unspliced .= ".rm_unspliced_10000000.gff3";
	if (-e $unspliced)
	{
	    my $new = $out;
	    $new =~ s/\.gff3$//;
	    $new .= ".rm_unspliced_10000000.gff3";
	    $cmd = "cp $unspliced $new";
	    system($cmd);
	}
	else
	{
	    print "warning: no unspliced file.$unspliced doesnt exist\n"; 
	}
    }
    
    
    if ($o_param->IsDefined('verbose'))
    {
	print "Nb of intron    = $global_intron_nb\n";
	print "Nb of intron NR =  " . scalar(keys(%h_intron_nr))."\n";
	print "Nb of intron NR with an incompatibility = " . scalar(keys(%h_incoherent_intron)). "\n";
	print "Nb of incompatibilities = " . scalar(keys(%h_incoherent_couple)). "\n";
	print "Nb of incompatibilities whith diff occ = " . scalar(keys(%h_incoherent_couple_nbdiff)). "\n";
	print "Nb intron to remove =" . scalar(keys(%h_toremove))."\n";
	print "Nb trancripts to remove: ". scalar(keys(%h_transcrit_toremove))."/".scalar(keys(%h_transcrit_list))."\n";
    }
    
    if (!$o_param->IsDefined('noclean'))
    {
	unlink("$out.ratio.histo") if ($o_param->IsDefined('histo'));
	unlink("$out.ratio") ;
	unlink ($out.".intron.gff3");
	unlink ("$intronnrfile", "$intronnrfile.intersection" , "$intronnrfile.intersection.name", "$intronnrfile.intersection.name.sort");	
	unlink($out.".listtranscrittoremove");
    }

    
    
}



sub MySort
{
    my ($occa, $namea ) = $a =~ /^(\d+)_(.+)$/;
    my ($occb, $nameb) = $b =~ /^(\d+)_(.+)$/;
    if ($occa != $occb)
    {
	return ($b <=> $a)
    }
    else
    {
	return $a cmp $b;
    }
   
}


sub Usage
{
    print STDERR<<END
Filter intron incoherences in EST_match/match_part gff3 file.
If incoherence, keep the introns with the most occurences. 
All the alignments of a transcript with at least one  intron incoherence are removed.
If ---keep_unspliced and input file .rm_unspliced_10000000.gff3 exists, copy it to out .rm_unspliced_10000000.gff3
$0
Mandatory
   --in     filename    gff3 formatted file (has to end by 'gff3') 
   --out    filename    file name           (has to end by 'gff3') 
Optional
   --keep_unspliced
   --verbose            To print count information
   --parent  string     default="EST_match"
   --child   string     default="match_part"
   --histo
   --noclean
END
}
