#!/usr/bin/perl

#
# $Id
#

# Copyright INRA

# Erika.Sallet@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod 

=cut


BEGIN
{
	$ENV{LANG} = "C";

	use Cwd 'abs_path';
	use File::Basename;
	my ($dirprg) =  dirname(abs_path($0));
	unshift(@INC,"$dirprg/../../../corelib");
}

use strict;
use Carp;
use ParamParser;
use General;
use GeneralBioinfo;
use Data::Dumper;

my $MINLEN = 25;

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG',\&Usage,'in=s','minlen=s','backup','do_not_remove_unspliced');

	my $ext_bk = time.".$$.filt";

    $o_param->SetUnlessDefined('minlen',$MINLEN);
    $o_param->AssertDefined('in');
	exit if ( -e $o_param->Get('in') && -z $o_param->Get('in') ); # rien a filtrer 
    $o_param->AssertFileExists('in');

    my $minlen = $o_param->Get('minlen');
	my $remove_unspliced = ( $o_param->IsDefined('do_not_remove_unspliced') ) ? &FALSE : &TRUE;

	my $match;
	my %h_res=();
	my @a_hit=();
	my $f_in=$o_param->GetStreamIn('in');
	my $outfile = $o_param->Get('in').".$ext_bk";
	my $f_out  = &GetStreamOut($outfile);
	while(my $l=<$f_in>)
	{
		chomp($l);
		&ParseGFF3Line(\$l,&TRUE,\%h_res);
		if ( $h_res{type} eq 'EST_match' )
		{
			&Filter($f_out,$match,\@a_hit,$minlen,$remove_unspliced);
			@a_hit=();
			$match=$l;
		}
		else
		{
			push(@a_hit,$l);
		}
	}
	&Filter($f_out,$match,\@a_hit,$minlen,$remove_unspliced);
	$f_in->close;
	$f_out->close;

	if ( $o_param->IsDefined('backup') )
	{
		system("cp ".$o_param->Get('in'). " $outfile.ori");
	}
	system("mv $outfile ".$o_param->Get('in'));
}

# je ne regarde que les exon extremes
# j'edite les frontieres du match si besoin
sub Filter
{
my($f_out,$match,$ra_hit,$minlen,$remove_unspliced)=@_;

	my %h_min=();
	my %h_max=();
	my %h_match=();
	my $start_match = 0;
	my $end_match = 0;
	if ( scalar(@$ra_hit) > 1 ) # il y a deux hits
	{
		&ParseGFF3Line(\$$ra_hit[0],&FALSE,\%h_min);
		if ( $h_min{len} < $minlen )
		{
			shift(@$ra_hit); # je zappe le premier exon
			&ParseGFF3Line(\$$ra_hit[0],&FALSE,\%h_min); # je recalcule le nouveau start a partir du nouveau premier exon
			$start_match=$h_min{start};
			if( scalar(@$ra_hit) == 1 ) # si apres la suppression d'un exon il en reste qu'un alors il faut aussi editer le end
			{
				&ParseGFF3Line(\$$ra_hit[0],&FALSE,\%h_max);
				$end_match=$h_max{end};
			}
		}
		&ParseGFF3Line(\$$ra_hit[$#$ra_hit],&FALSE,\%h_max);
		if ( $h_max{len} < $minlen )
		{
			pop(@$ra_hit);
			if ( scalar(@$ra_hit) ) # on peut avoir deux exons minuscules et les supprimer tous les deux
			{
				&ParseGFF3Line(\$$ra_hit[$#$ra_hit],&FALSE,\%h_max);
				$end_match=$h_max{end};
				if( scalar(@$ra_hit) == 1 )
				{
					&ParseGFF3Line(\$$ra_hit[0],&FALSE,\%h_min);
					$start_match=$h_min{start};
				}
			}
		}
	}
	return if ( ($remove_unspliced && scalar(@$ra_hit) <= 1) || scalar(@$ra_hit) == 0 );

	&ParseGFF3Line(\$match,&FALSE,\%h_match);
	$h_match{start} = $start_match if ( $start_match );
	$h_match{end}   = $end_match   if ( $end_match );

	printf $f_out "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",$h_match{seqid}, $h_match{source}, $h_match{type}, $h_match{start}, $h_match{end}, $h_match{score}, $h_match{strand}, $h_match{phase}, $h_match{attributes};
	for(my $i=0;$i<scalar(@$ra_hit);$i++)
	{
		print $f_out "$$ra_hit[$i]\n";
	}
}



sub Usage
{
    print STDERR<<END
Remove small end exons (the gmap param --trimendexons doesn't work)
$0
Mandatory
   --in     filename    gff3 formatted file (match/match_part)
   --minlen integer     [$MINLEN] minimum length of terminal exons
Optional
   --backup             keep the unfiltered version
   --do_not_remove_unspliced   deactivate the filter on multiple exons
END
}
