#!/usr/bin/perl

use strict;
use ParamParser;
use IO::File;
use General;

	my %h_color = (
		21 => '0,0,255',
		22 => '0,0,139',
		23 => '58,95,205',
		27 => '0,255,0',
		28 => '127,255,0',
		29 => '0,255,127',
		36 => '255,0,0',
		37 => '255,99,71',
		38 => '240,128,128',
		39 => '208,32,144',
		40 => '238,130,238',
		41 => '219,112,147',
		4  => '255,255,0',
		60 => '255,165,0', # orange
		'21-22-23' => '0,0,255',
		'27-28-29' => '0,255,0',
		'36-37-38' => '255,0,0',
		'39-40-41' => '208,32,144',
		'30-31-32' => '255,99,71',
		'33-34-35' => '238,130,238',
		'51-52-53' => '0,255,255',     # cyan
		'ZI.1'       => '255,236,139', # yellow
		'ZI.2'       => '255,236,139',
		'ZI.3'       => '255,236,139',
		'ZI.1-2-3'   => '255,236,139',
		'ZIId.1'     => '255,160,122', # salmon
		'ZIId.2'     => '255,160,122',
		'ZIId.3'     => '255,160,122',	
		'ZIId.1-2-3' => '255,160,122',
		'ZIIp.1'     => '255,28,174',  # rose
		'ZIIp.2'     => '255,28,174',	
		'ZIIp.3'     => '255,28,174',	
		'ZIIp.1-2-3' => '255,28,174',
		'IZ.1'       => '191,62,255',  # violet
		'IZ.2'       => '191,62,255',
		'IZ.3'       => '191,62,255',
		'IZ.1-2-3'   => '191,62,255',
		'ZIII.1'     => '104,34,139', # dark violet
		'ZIII.2'     => '104,34,139',
		'ZIII.3'     => '104,34,139',
		'ZIII.1-2-3' => '104,34,139',
		'ZV.1'       => '35,35,142', #	dark blue
		'ZV.2'       => '35,35,142',	
		'ZV.3'       => '35,35,142',
		'ZV.1-2-3'   => '35,35,142',
		'ZVcont.1'     => '77,77,255',  #	 clear blue
		'ZVcont.2'     => '77,77,255',
		'ZVcont.3'     => '77,77,255',
		'ZVcont.1-2-3' => '77,77,255',
	);


	my $DEF_minus_color = '130,130,130'; # gris	
        my $DEF_plus_color  = '0,0,255'; # bleu

use constant DEFAULT_PREV_SCALE  => 1;
use constant DEFAULT_THRESHOLD   => 1000000000000000;
use constant DEFAULT_MUL_LOG_VAL => 10;

MAIN:
{
	my $o_param =  New ParamParser;
    $o_param->SetUsage(my $usage = sub {&Usage();});

	$o_param->Update("GETOPTLONG","I", ("replicon=s", "position_file=s", "wig_file=s", "len=s", "prev_scale=i", "threshold=i",
										"mul_log_val=i", "log", "color=s", 'strand=s', 'negative')); 
	
	$o_param->AssertFileExists('position_file');
	$o_param->AssertDefined('wig_file', 'replicon');
	my $position_file =	$o_param->Get('position_file');	
	my $wig_file      = $o_param->Get('wig_file');							
	my $len           = $o_param->Get('len');			
	my $replicon      = $o_param->Get('replicon');
	my $negative = 0;
	$negative = 1 if ($o_param->IsDefined('negative'));

	$o_param->SetUnlessDefined('prev_scale', &DEFAULT_PREV_SCALE );	
	$o_param->SetUnlessDefined('mul_log_val',  &DEFAULT_MUL_LOG_VAL );
	my $prev_scale  = $o_param->Get('prev_scale');
	my $mul_log_val = $o_param->Get('mul_log_val');	

	
	
	$o_param->AssertAllowedValue('strand', 'minus', 'plus') if ($o_param->IsDefined('strand'));
	my $wanted_strand = "both";
	$wanted_strand = $o_param->Get('strand') if ($o_param->IsDefined('strand'));
	
	my $threshold = &DEFAULT_THRESHOLD;
	$threshold    = $o_param->Get('threshold') if ($o_param->IsDefined('threshold'));
	
	my @a_wig_forward;
	my @a_wig_reverse;
	
	my $max1_val = 0;	my $max2_val = 0;
	$max1_val = &PrintStrand($position_file, \@a_wig_forward, $o_param, "+", $replicon, $prev_scale,$threshold, $mul_log_val) if ($wanted_strand eq "both" || $wanted_strand eq "plus");
	$max2_val = &PrintStrand($position_file, \@a_wig_reverse, $o_param, "-", $replicon, $prev_scale,$threshold, $mul_log_val) if ($wanted_strand eq "both" || $wanted_strand eq "minus");
	my $max      = General::Max($max1_val, $max2_val);
	
	my $plus_color;
	my $minus_color;
	if ($o_param->IsDefined('color'))
	{
	    $plus_color  = $o_param->Get('color');
	    $minus_color = $o_param->Get('color');
	}
	elsif ($o_param->IsDefined('len') && defined $h_color{$o_param->Get('len')})
	{
	    $plus_color  = $h_color{$o_param->Get('len')};
	    $minus_color = $DEF_minus_color;
	}
	else
	{
	    $minus_color = $DEF_minus_color;
	    $plus_color  = $DEF_plus_color;
	}
	
	&PrintWig($wig_file, \@a_wig_forward, \@a_wig_reverse, $replicon, $max, $plus_color, $minus_color, $wanted_strand, $negative);

}


sub PrintWig
{
	my ($wig_file, $ra_wig_forward, $ra_wig_reverse, $replicon, $max_val, $plus_color, $minus_color, $strand, $negative) = @_;

	my $f_wig = General::GetStreamOut($wig_file);
	
	if ($strand eq "both" || $strand eq "plus")
	{
		
		# Print forward data
		print $f_wig "track color=".$plus_color."\n";
		print $f_wig "variableStep chrom=$replicon\n";
		print  $f_wig "0\t1\n0\t$max_val\n" if ($strand eq "both");
		foreach my $line (@$ra_wig_forward)
		{
			print $f_wig $line;
		}
	}

	if ($strand eq "both" || $strand eq "minus")
	{
		# Print reverse data
		print $f_wig "track color=".$minus_color."\n";
		print $f_wig "variableStep chrom=$replicon\n";
		if ($negative)
		{
		    print  $f_wig "0\t-1\n0\t-$max_val\n" if ($strand eq "both");
		}
		else
		{
		    print  $f_wig "0\t1\n0\t$max_val\n" if ($strand eq "both");;
		}
		foreach my $line (@$ra_wig_reverse)
		{
			print $f_wig $line;
		}
	}
	
	$f_wig->close();

	return;
		
}

sub PrintStrand
{
	my ($position_file, $ra_wig,  $o_param, $wanted_strand, $replicon, $prev_scale, $threshold, $mul_log_val) = @_;
	
	my $max_val = 0;
	my $f_pos = &General::GetStreamIn($position_file) ;

	
	while (my $line = <$f_pos>)
	{

		my @a_ = split(/\t/, $line);
		my $pos    = $a_[3];
		my $val    = $a_[5] * $prev_scale;
		my $strand = $a_[6];
		
		if ($o_param->IsDefined('log'))
		{
			$val = sprintf ("%0.3f", log($val)/log(2));
			$val = $val*$mul_log_val; 
		}
		else
		{
			$val = $threshold if ($val > $threshold);
		}
		$val = "-".$val if ($strand eq '-' && $o_param->IsDefined('negative'));
		#print $f_wig "$pos\t$val\n" if ($strand eq $wanted_strand);
		push (@$ra_wig, "$pos\t$val\n") if ($strand eq $wanted_strand);

		$max_val = abs($val) if (abs($val)> $max_val);
	}
	$f_pos->close();

	return $max_val;
}


=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
    print STDERR<<END
$0     
Convert position file into wig file

Mandatory
  --replicon  Gm.a
  --position_file file.position.gff3
  --wig_file file.wig output file

Optional
  --prev_scale  1000 [Default 1]
  --threshold   300  [Default +inf]
  --mul_log_val integer [Default 10]
  --color  string rgb color code. Example: '0,0,255'
  --len  '20-21-22' (to use preselected color for this len)
  --log Put in log
  --strand minus/plus [Defaut both]
  --negative           To create negative value for features in minus strand
  --help
END
      ;
}
