# See http://www.ebi.ac.uk/ena/WebFeat/ for the genbank tags 
# See http://www.sequenceontology.org/ for the sequence ontology terms

# %H_CL_FEATURES connects the Genbank features and the classes used in BioFileConverter.
# All the classes should derive from Locus
# Several features may correspond to a class, thus the value is a "set of values", implemented with a perl hash
# key   = the classes derived from Locus (Locus::XXX)
# Value = sets of names

%H_CL_FEATURES  = ( 'Locus::Gene'       => { gene => 1 },
					'Locus::Mrna'       => { mRNA => 1 },
					'Locus::Exon'       => { exon => 1 },
					'Locus::CdsRegion'  => { CDS  => 1 },
					'Locus::MiscRegion' => { misc_feature => 1, prim_transcript => 1, rep_origin => 1, mobile_element => 1, gap => 1, assembly_gap => 1, precursor_RNA => 1 },
					'Locus::Ncrna'      => { ncRNA => 1, rRNA => 1, tRNA => 1, tmRNA => 1, misc_RNA => 1 },
					'Locus::Repeat'     => { repeat_region => 1, LTR => 1 },
					'Locus::UtrRegion'  => { "3'UTR" => 1, "5'UTR" => 1 }
				);

# %H_QUALIFIERS gives, for each genbank feature the list of possible qualifiers
# Only the list of qualifiers is used yet, the type of each qualifier is not used (but may be used in the future)
%H_QUALIFIERS = ( gene => {   'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'operon' => "text", 'product' => "text", 'pseudogene' => "TYPE",
								'phenotype' => "text", 'standard_name' => "text", 'trans_splicing' => 1 },
					mRNA => {   'allele' => "text", 'artificial_location' => "[artificial_location_value]", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'operon' => "text", 'product' => "text", 'pseudogene' => "TYPE",
								'standard_name' => "text", 'trans_splicing' => 1, 'transcript_id' => "text" },
					exon => {   'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'EC_number' => "text", 'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'number' => 'unquoted text', 'old_locus_tag' => "text", 'product' => "text", 'pseudogene' => "TYPE",
								'standard_name' => "text", 'trans_splicing' => 1 },
					CDS =>  {   'allele' => "text", 'artificial_location' => "[artificial_location_value]", 'citation' => '[number]', 'codon_start' => '1/2/3', 'db_xref' => "<database>:<identifier>",
								'EC_number' => "text", 'exception' => "[exception_value]", 
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'number' => 'unquoted text', 'old_locus_tag' => "text", 'operon' => 'text', 'product' => "text", 'protein_id' => "<identifier>",'pseudogene' => "TYPE",
								'ribosomal_slippage' => 1, 'standard_name' => "text", 'translation' => 'text', 'transl_except' => '(pos:<base_range>,aa:<amino_acid>)',
								'transl_table' => '<integer>', 'trans_splicing' => 1 },
					mobile_element => { 'mobile_element_type' => "text", 'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'number' => 'unquoted text (single token)', 'old_locus_tag' => "text",
								'rpt_family' => "text", 'rpt_type' => '<repeat_type>', 'standard_name' => "text" },
					misc_feature => { 'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'number' => 'unquoted text (single token)', 'old_locus_tag' => "text",
								'phenotype' => "text", 'product' => "text", 'pseudogene' => "TYPE", 'standard_name' => "text" },
					gap => {    'estimated_length' => 'unknown or <integer>', experiment => '[CATEGORY:]text', 'inference' => '[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]',
								'map' => "text", 'note' => "text" },
					assembly_gap => { 'estimated_length' => 'unknown or <integer>', gap_type => "TYPE", linkage_evidence => "TYPE" },
					prim_transcript => { 'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'operon' => "text", 'standard_name' => "text" },
				    rep_origin => { 'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
									'experiment' => "[CATEGORY:]text", 'gene' => 'text', 'gene_synonym' => "text",
									'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
									'note' => "text", 'old_locus_tag' => "text", 'standard_name' => "text" },
					ncRNA => {  'ncRNA_class' => "TYPE", 
								'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'operon' => "text", 'product' => "text", 'pseudogene' => "TYPE",
								'trans_splicing' => 1, 'standard_name' => "text", 'transcript_id' => "text" },
					rRNA => {   'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'operon' => "text", 'product' => "text", 'pseudogene' => "TYPE",
								'standard_name' => "text", 'transcript_id' => "text" },
					tRNA =>  {  'allele' => "text", 'anticodon' => '(pos:<location>,aa:<amino_acid>)', 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'operon' => "text", 'product' => "text", 'pseudogene' => "TYPE",
								'trans_splicing' => 1, 'standard_name' => "text" },
					tmRNA => {  'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'product' => "text", 'pseudogene' => "TYPE", 'standard_name' => "text",
								'tag_peptide' => '<base_range>' },
					misc_RNA=>{ 'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'operon' => 'text', 'product' => "text", 'pseudogene' => "TYPE", 'standard_name' => "text" },
					precursor_RNA => { 'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'operon' => 'text', 'product' => "text", 'pseudogene' => "TYPE", 'standard_name' => "text", 'trans_splicing' => 1 },
					repeat_region => { 'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text", 'rpt_family' => "text", 'rpt_type' => '<repeat_type>', 
								'rpt_unit_range' => '<base_range>', 'rpt_unit_seq' => "text", 'stallite' => "<satellite_type>[:<class>][ <identifier>]",
								'standard_name' => "text" },
					LTR =>   {  'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text",'standard_name' => "text" },
					"3'UTR" => {'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text",'standard_name' => "text", 'trans_splicing' => 1 },
					"5'UTR" => {'allele' => "text", 'citation' => '[number]', 'db_xref' => "<database>:<identifier>",
								'experiment' => "[CATEGORY:]text", 'function' => "text", 'gene' => 'text', 'gene_synonym' => "text",
								'inference' => "[CATEGORY:]TYPE[ (same species)][:EVIDENCE_BASIS]", 'locus_tag' => "text", 'map' => "text",
								'note' => "text", 'old_locus_tag' => "text",'standard_name' => "text", 'trans_splicing' => 1 } );

# %H_SO2FEATURE connects Sequence Ontology terms with Genbank/Embl/Ddbj features
# Is is used when reading/writing genbank/embl/ddbj files
# key  = A sequence ontology term
# value= 1/ A Genbank/Embl/Ddbj feature
#        2/ mobile_element followed by _ followed by a mobile element type
%H_SO2FEATURE = (  three_prime_UTR => "3'UTR",
					five_prime_UTR  => "5'UTR",
					tRNA_primary_transcript => "precursor_RNA",
					non_coding_gene => 'gene',
					coding_gene => 'gene',
					insertion_site => 'misc_feature',
					mobile_element_insertion => 'mobile_element',
					region => 'misc_feature',
					insertion_sequence => 'mobile_element_insertion sequence',
					DNA_transposon => 'mobile_element_transposon',
					retrotransposon => 'mobile_element_retrotransposon',
					integron => 'mobile_element_integron',
					non_LTR_retrotransposon => 'mobile_element_non-LTR retrotransposon',
					SINE_element => 'mobile_element_SINE',
					LINE_element => 'mobile_element_LINE',
					MITE => 'mobile_element_MITE',
					pseudogene => 'misc_feature' );

# %H_FEATURE2SO connects Genbank/Embl/Ddbj features with Sequence Ontology terms
# Is is used when reading/writing genbank/embl/ddbj files
#  It IS NOT the reverse of %H_SO2FEATURE, because the former is a n21 relationship
# key  = A Genbank/Embl/Ddbj feature
# value= A sequence ontology term
%H_FEATURE2SO = (	"3'UTR" => "three_prime_UTR",
					"5'UTR" => "five_prime_UTR",
					precursor_RNA => "tRNA_primary_transcript",
					'mobile_element' => 'region',
					'mobile_element_insertion sequence' => 'insertion_sequence',
					'mobile_element_transposon' => 'DNA_transposon',
					'mobile_element_retrotransposon' => 'retrotransposon',
					'mobile_element_integron' => 'integron',
					'mobile_element_non-LTR retrotransposon' => 'non_LTR_retrotransposon',
					'mobile_element_SINE' => 'SINE_element',
					'mobile_element_LINE' => 'LINE_element',
					'mobile_element_MITE' => 'MITE' );

# key   = some genbank features
# value = some legal types for this feature, will be integrated to /note qualifiers
%H_TYPES = ( misc_feature => { 'region' => 1, 'insertion_site' => 1} );
