#
# $Id$
#
 
package Submit;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

AnnotatedSeqFileReader::UsingBioPerl - parse a file and load an AnnotatedSeq object, using the BioPerl API
                                       Currently a base class for genbank and embl file formats
                                       See www.bioperl.org

=head1 SYNOPSIS
	See AnnotatedSeq

=head1 DESCRIPTION

=cut

use strict;
use warnings;
use Carp;

use Error qw(:try);
use General;

use base qw( LipmSingleton );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, initialize the object, do not use directly, use New method instead
 Usage        : my $o_submit = New Submit::XXX('pre'=>pre_script.pl,'post'=>post_script.pl);

 Returns      : none
 Args         : %h_param 
                   Optional: pre      A file containing perl code to be executed by ApplyRules BEFORE anything else
                   Optional: post     A file containing perl code to be executed by ApplyRules AFTER everything else

=cut

sub _Init { 
	my $self = shift;
	my %h_param = @_;
	$self->SUPER::_Init(@_);

	$self->_Set('__pre',$h_param{'pre'})   if exists($h_param{'pre'});
	$self->_Set('__post',$h_param{'post'}) if exists($h_param{'post'});
	$self->_Set('_protein-namespace',$h_param{'protein-namespace'}) if exists($h_param{'protein-namespace'});
};

# Implementing Submit ApplyRules
#        1/ Call _ApplyRules, implemented in the derived class, which applies rules to MODIFY data
#        2/ Call _CheckRules, implemented in the derived class, which applies rules to CHECK data for quality
sub ApplyRules 
{
	my $self = shift;
	$self -> _ApplyRules(@_);
	$self -> _CheckRules(@_);
};

sub _ApplyRules{};
sub _CheckRules{};

=head2 function _ApplyPre,_ApplyPost
 Title    : _ApplyPre,_ApplyPost
 Function : Should be called by the derived class ApplyRules BEFORE or AFTER the code of ApplyRules
            Return the code to be executed through an eval, thus ApplyRules should look like:
            eval $self->_ApplyPre(); die $@ if $@;
            ...
            eval $self->_ApplyPost(); die $@ if $@;
 Access   : Protected

=cut
sub _ApplyPre
{
	my $self = shift;
	return $self -> __ApplyExternal($self->_Get('__pre'));
}
sub _ApplyPost
{
	my $self = shift;
	return $self -> __ApplyExternal($self->_Get('__post'));
}
sub __ApplyExternal
{
	my $self = shift;
	my ($script) = @_;
	my $code = "";
	if ( defined ($script) && $script ne '' )
	{
		my $fh_script = GetStreamIn($script);
		$code = join ( "", <$fh_script> );
		print STDERR "\nNow executing $script\n" if ( $code ne '' );
	}

	return $code;
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
