#
# 	emmanuel.courcelle@toulouse.inra.fr
#	Created: Nov 18, 2010
#	$Id: LineByLine.pm 770 2011-12-16 14:38:49Z manu $
#
# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

	LineByLine - Class to read 

=head1  SYNOPSIS

	my $o_ndr = New Bio::Narcisse::Utils::NarcisseDataReader::LineByLine();
	$o_ndr -> Prepare ( -from => $file );
	while ( my $rh_line = $o_ndr->FetchNextLine() )
	{
		...
	}

=head1 DESCRIPTION
	                
    Compared to NarcisseDataReader, this class is *VERY* primitive:
       - The cache is NOT used
       - Prepare does accept any parameter except -from
       - The file  is NOT read entirely into the memory
       - The only way to read the file is to continuously call the method FetchNextLine()

         This is surely less fast, but the memory consumption lowers dramatically
         This object should be used when you have to read huge files (> 50Mb), as in Pairwise.pm,
         where we sometimes have to read files as long as 1 Gb or more.

=cut




# NO MARCHE SUREMENT PAS - EN DEVELOPPEMENT - DO NOT USE YET

package DataReader::LineByLine;

use strict;
use warnings;

use base qw( DataReader );

use POSIX;  # for the ceil function
use Data::Dumper;
use Carp;
use LipmError::IOException;
use LipmError::ParameterException;

use General;

use constant MAX_CHROM_SIZE => 4000000000;
use constant ARRAY => 0;
use constant HASH  => 1;
use constant SLICE_UNIT =>  100000; # 100kb is the unit slice size 

# Column separator
our $SEP = "\t";

# A cache for file content, common to all objects of this class
# The key is the file name (see the switch -from in method Prepare)
# The value is an array of hashes, 1 hash / line

#our %H_DATA_CACHE = ();

#----------------------------------
#          Reading the file
#---------------------------------
=head2 _Init
 The constructor - Just init the private fields and call the mother class constructor
=cut
sub _Init
{
	my $self = shift;
	$self -> _Set ( '__fh_file' => '' );
	$self->SUPER::_Init( @_ );
}

=head2 Procedure Prepare

 Usage        : $o_obj->Prepare( -from => $file )
 Procedure    : The file is opened, and only the headers are read and parsed
                                     
=cut
sub Prepare
{
	my $self = shift;
	my %h_args = (
        -from   => '-from option',	# dummy name, useful for error message generation
        @_ );
        
 	my $file = $h_args{ -from };

	$self->_Set( _file => $file,
				 _case_sensitive => $h_args{ -case_sensitive } );

	# Clearing the return structures we are preparing the object for new data to retrieve
	# SANS DOUTE PAS UTILE MAIS NE FAIT PAS DE MAL (manu)
	$self->_Set( '_ra_return' 		=> [],
	             '_rh_return' 		=> {},
	             '_ra_sorted_keys'  => []
	            );

	# Open the file
	my $fh_file = new IO::File( $file );
	$self -> _Set ( '__fh_file' => $fh_file );
	if( !defined $fh_file )
	{
		throw LipmError::IOException( $file, 'f', -depth => 1 );
	}
	
	my $ra_header_line;
	$ra_header_line = $self -> __ReadLine();
	
	# If there is an index, we skip the index (TODO - l'utiliser ?)
	if ( $ra_header_line -> [0] =~ /\@/ )
	{
		$ra_header_line = $self -> __ReadLine();
	}
	$self -> _RetrieveHeaderFields ( $ra_header_line );
				
	return;
}

=head2 Procedure FetchNextLine

 Usage        : my $rh_line = $self -> FetchNextLine();
 Prerequisite : The file must be opened
 Procedure    : Reads a line of the Narcisse formated file and returns a ref to an array
                Empty lines and lines starting with ';' are skipped
 Args         : None
 Returns 	  : a reference to an array containing the data
=cut
sub FetchNextLine
{
	my $self       = shift;
	my $ra_row     = $self -> __ReadLine();
	my $ra_headers = $self -> _Get('_ra_headers');
	if ( defined $ra_row )
	{
		my %h_row = ();
		@h_row{ @$ra_headers } = @{$ra_row};
		return \%h_row;
	}
	else
	{
		return undef;
	}
}
	
=head2 Procedure __ReadLine

 Usage        : my $ra_line = $self -> __ReadLine();
 Prerequisite : The file must be opened
 Procedure    : Reads a line of the Narcisse formated file and returns a ref to an array
                Empty lines and lines starting with ';' are skipped
 Args         : None
 Returns 	  : a reference to an array containing the data
                undef if there are no more data to read
=cut
sub __ReadLine
{
	my $self = shift;
	
	my $fh_file = $self -> _Get ( '__fh_file' );
	my @a_row = ();
	my $invalid_data = &TRUE;
	while( my $line=<$fh_file> )
	{
		next unless ($line =~ /\S+/);
		next if ($line =~ /^;/);
		
		chomp($line);
		@a_row = split( /$SEP/, $line );
		$invalid_data = &FALSE;
		last;
	}
	return (($invalid_data == &TRUE) ? undef : \@a_row);
}


=head2 Procedure DESTROY

 Usage     : The destructor
 Procedure : Close the file
 Args      : None
=cut

sub DESTROY
{
	my $self = shift;
	my $fh_file = $self -> _Get ( '__fh_file' );
	$fh_file -> close() unless ( $fh_file eq '' );
}
1;
