#
# $Id: Rna.pm 1438 2015-04-20 13:33:57Z carrere $
#

package Locus::Rna;

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

Locus::Rna - describes a Rna, may be a Mrna or a Ncrna 
             Locus::rna is an abstract class, SHOULD NOT BE INSTANTIED

=head1 SYNOPSIS

=head1 DESCRIPTION

 A Rna extends the Locus class, including a Locus::ExonSet to the locus
 The parent may be a Locus::Gene object or a AnnotatedSeq object (see Mrna::SetParent and Ncrna::SetParent)

=cut

use strict;
use warnings;
use Carp;
use Bio::Seq;
use Data::Dumper;

use General;
use Locus;

use base qw( Locus );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 1438 $ =~ /\d+/g); $r[0]};
}

=head2 procedure _Init

 Title        : _Init
 Function     : Constructor, see Locus::_Init for the details
 Usage        : my $o_rmna = New Locus::Mrna ( %h_param );
                my $o_rmna = New Locus::Mrna ( %h_param, $o_locus );

=cut

sub _Init
{
	my $self = shift;
	$self->SUPER::_Init(@_);
}

=head2 procedures Select,Deselect

 See Locus.pm for the generic documentation
 we select/deselect the Locus::Cds, the Locus::ExonSet and the Locus::Utr objects

=cut
sub Select
{
	my $self = shift;
	my ( $inside ) = @_;
	$inside = &TRUE unless (defined $inside);
	if ( $inside )
	{
		$self->GetExonSet()-> Select() if ( defined ( $self-> { __o_exonset } ) );
	}
	delete $self -> { __deselected };
}

sub Deselect
{
	my $self   = shift;
	my ( $inside ) = @_;
	$inside = &TRUE unless (defined $inside);
	if ( $inside )
	{
		$self->GetExonSet()-> Deselect() if ( defined ( $self-> { __o_exonset } ) );
	}
	$self -> { __deselected } = &TRUE;
}

=head2 function GetTranscriptId

 Title : GetTranscriptId
 Usage : my $id = $o_mrna -> GetTranscriptId();
=cut
sub GetTranscriptId
{
	my $self = shift;
	my $as_string = @_;
	return ($as_string) ? $self -> GetAsString ( 'transcript_id' ) : $self -> Get ( 'transcript_id' );
}

sub GetTranscriptName
{
	my $self = shift;
	my $as_string = @_;
	return ($as_string) ? $self -> GetAsString ( 'Name' ) : $self -> Get ( 'Name' );
}



=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;


